# Grégory Coutable 2022
# Partage sous licence Creative Commons :
# https://creativecommons.org/licenses/by-sa/3.0/fr/
#
# Version 1

import pygame
from pygame.locals import *
#pour le rendre dispo de n'importe où
import os
pathname = os.path.dirname(__file__)

class GUIhanoi (object):
    
    def __init__(self, n):
        """
        Initialise une fenètre du jeu de Hanoi pour n pièce.
        """
        
        #initialisation de pygame
        pygame.init()

        #adaptation de l'unité graphique au nombre de piece du jeu
        self.nbPiece = n
        if n < 9 :
            self.uGraph = 20
        elif n < 16 :
            self.uGraph = 15
        else:
            self.uGraph = 10
            
        #hauteur d'une tour
        self.hTour = self.uGraph * (n * 2 + 7)
        #largeur d'une tour
        self.wTour = self.uGraph * (n * 2 + 5)

        self.hBandeau = 50
        self.rectBandeau = pygame.Rect(0, 0, 3*self.wTour, self.hBandeau)
        
        self.rectPiece = [[pygame.Rect(i*self.wTour + self.uGraph * (2 + self.nbPiece), self.hBandeau + (3 + 2*self.nbPiece - 2*num)*self.uGraph, self.uGraph, 2*self.uGraph) for num in range(self.nbPiece)] for i in range(3)]
        
        #on définit la fenêtre de base de notre jeu
        self.fenetre = pygame.display.set_mode((3*self.wTour, self.hTour + self.hBandeau))
        #un titre sur cette fenêtre
        pygame.display.set_caption("Les tours de Hanoï")
        
        self.refresh([[], [], []], "Les tours de Hanoï")

    def _dessinePiece(self, num, rect):
        """
        Dessine une pièce en fonction de son numéro (1 est la plus petite) et la positionne centrée sur rect.
        rect est un objet de la classe Rect de pygame.
        """
        
        case = pygame.Surface((self.uGraph * (num * 2 + 1), 2 * self.uGraph))
        case.fill(self._color(1 + self.nbPiece - num, self.nbPiece))
        
        caseRect = case.get_rect()
        caseRect.center = rect.center
       
       
        self.fenetre.blit(case, caseRect)

    def _dessineSocle(self, x0):
        """
        Déssine le socle de la tour qui portera les pièces.
        x0 est le décalage à l'origine sur l'axe x.
        """
        #dessine le support des pièces
        #le socle
        
        socle = pygame.Surface((self.wTour - 2 * self.uGraph, self.uGraph))
        socle.fill(pygame.Color('brown'))
        self.fenetre.blit(socle, (x0 + self.uGraph, self.hBandeau + self.hTour - 2 * self.uGraph))

        #le pic
        
        pic = pygame.Surface((self.uGraph, self.hTour - 6 * self.uGraph))
        pic.fill(pygame.Color('brown'))
        self.fenetre.blit(pic, (x0 + self.uGraph * (2 + self.nbPiece), self.hBandeau + 4 * self.uGraph))

    @staticmethod
    #un joli dégradé de bleu
    def _color(n, nMax):
        b = 255
        g = r = (n - 1) * 255 // nMax
        return pygame.Color("#%02x%02x%02x"%(r,g,b))

    def _placeElements(self, g, t):
        #Limitation de vitesse de la boucle
        #pygame.time.Clock().tick(30)
        #réinitialisation du fond
        #self.fenetre.fill(0x50c63d)
        self.fenetre.fill(0xfdf4e3)
        #le socle et le pic
        for i in range(3):
            self._dessineSocle(i * self.wTour)
        #les pièces
        for x in range(3):
            for y in range(len(g[x])):
                self._dessinePiece(g[x][y], self.rectPiece[x][y])
        #zone de texte
        #Une police
        police = pygame.font.SysFont("Arial Black",20)
        #un texte
        texte = police.render(t, True, pygame.Color("black"))
        #pour centrer le texte
        rectTexte = texte.get_rect()
        rectTexte.center = self.rectBandeau.center
        self.fenetre.blit(texte, rectTexte)
        #Rafraîchissement de l'écran
        pygame.display.update()

    def refresh(self, g, t = "", *nc):
        """
        Cette méthode rafraichie l'affichage du jeu conformément à la grille passée en argument.
        
        g est une liste contenant les 3 tours. Une tour est une structure de type list ou tout autre objet qui possède
        les méthodes __len__ et __getitem__, le rendant indexable.
        la première piece d'une tour (un int > 0) est a l'index 0 de la tour.
        
        t est un texte à afficher à destination des joueurs.
        
        *nc est un tupple contenant deux entiers optionnels colD et col A. Si nc est spécifié, cela déclenche une animation
        en déplaçant graphiquement pièce au sommet de la tour nD vers la tour nA.
        Attention cette animation est purement visuelle, g n'est pas modifiée par cette opération.
        """

            
        if len(nc) == 2:
            #demande animation
            colD, colA = nc
            
            nA, nD = len(g[colA]), len(g[colD]) - 1
            
            rectMouv = self.rectPiece[colD][nD]
            rectArrivee = self.rectPiece[colA][nA]
            
            #le rect va se déplacer entre ces deux positions, entrainant la piece avec lui
            #vers le haut
            while rectMouv.top > self.rectBandeau.bottom + self.uGraph :
                rectMouv.top -= 1
                self._placeElements(g, t)
            #vers la gauche ou la droite
            while rectMouv.left != rectArrivee.left :
                diff = rectArrivee.left - rectMouv.left
                rectMouv.left += diff // abs(diff)
                self._placeElements(g, t)
            #vers le bas
            while rectMouv.top != rectArrivee.top :
                rectMouv.top += 1
                self._placeElements(g, t)
                
            #on replace le rect a sa position initiale
            self.rectPiece[colD][nD] = pygame.Rect(colD*self.wTour + self.uGraph * (2 + self.nbPiece), self.hBandeau + (3 + 2*self.nbPiece - 2*nD)*self.uGraph, self.uGraph, 2*self.uGraph)
            
        else:
            #pas d'animation
            self._placeElements(g, t)

    def messCentre(self, mess):
        """
        Cette méthode permet d'afficher un message plein écran.
        """
        police = pygame.font.Font(os.path.join(pathname, "led.ttf"),80)
        texte = police.render(mess, True, pygame.Color("black"))
        #pour centrer le texte
        rectTexte = texte.get_rect()
        rectTexte.center = self.fenetre.get_rect().center
        self.fenetre.blit(texte,rectTexte)
        #Rafraîchissement de l'écran
        pygame.display.update()

    def waitClick(self):
        """
        Cette méthode attend l'action d'un joueur. Elle gère trois type d'actions :
            - demande fermeture de la fenètre : fermeture propre de la fenètre pygame et fin du programme python.
            - click sur la fenetre : retourne  un tuple qui référence la tour cliquée 0, 1 ou 2.
            - appui sur des touches.
        Une fois exécutée, on ne peut sortir de cette méthode que par l'une de ces trois actions.
        """
        continuer = True
        while continuer:
            #Limitation de vitesse de la boucle
            pygame.time.Clock().tick(30)
            
            for event in pygame.event.get():    #Attente des événements
                if event.type == QUIT:
                    continuer = False
                    pygame.quit()
                    exit()
                    
                if event.type == MOUSEBUTTONDOWN:
                    if event.button == 1:   #Si clic gauche
                        return event.pos[0] // self.wTour                    
                    
                if event.type == KEYDOWN:
                    touches = {K_RIGHT : '_R', K_LEFT : '_L', K_UP : '_U', K_DOWN : '_D', K_RETURN : '_E', K_BACKSPACE : '_B', K_ESCAPE : '_S'}
                    touche = event.key
                    if touche in touches:
                        return touches[touche]
                    return event.unicode
                    
                         
if __name__ == "__main__":
    import time
   
    GUI = GUIhanoi(5)
    time.sleep(2)
    GUI.refresh([[5,4], [3,2,1], []], "a vous de jouer !!")
    time.sleep(2)
    GUI.refresh([[5,4], [3,2,1], []], "Animation 1", 1, 2)
    GUI.refresh([[5,4], [3,2], [1]], "Animation 2", 1, 0)

    GUI.messCentre('GAGNE !!')
    

    
    while True :
        print(GUI.waitClick())

